@@ -27,7 +27,7 @@
 #define DUMP_STRCASE(_t) \
         case _t: { Log(("%s", #_t"")); break; }
 #define DUMP_STRCASE_DEFAULT_DWORD(_dw) \
-        default: { Log(("0x%08", (_dw))); break; }
+        default: { Log(("0x%08x", (_dw))); break; }
 
 #define DUMP_STRIF_INIT(_ps, _t) \
         const char * _pSep = (_ps); \
