@@ -37,16 +37,12 @@ VBoxDnDDropSource::VBoxDnDDropSource(VBoxDnDWnd *pParent)
       mdwCurEffect(0),
       muCurAction(DND_IGNORE_ACTION)
 {
-    int rc = VbglR3DnDConnect(&mDnDCtx);
-
-    LogFlowFunc(("rc=%Rrc\n", rc));
+    LogFlowFuncEnter();
 }
 
 VBoxDnDDropSource::~VBoxDnDDropSource(void)
 {
-    int rc = VbglR3DnDDisconnect(&mDnDCtx);
-
-    LogFlowFunc(("rc=%Rrc, mRefCount=%RI32\n", rc, mRefCount));
+    LogFlowFunc(("mRefCount=%RI32\n", mRefCount));
 }
 
 /*
