@@ -1,4 +1,4 @@
-/* $Rev: 102121 $ */
+/* $Rev: 103598 $ */
 /** @file
  * VBoxGuest - Linux specifics.
  *
@@ -55,7 +55,7 @@
 *********************************************************************************************************************************/
 /** The device name. */
 #define DEVICE_NAME             "vboxguest"
-/** The device name for the device node open to everyone.. */
+/** The device name for the device node open to everyone. */
 #define DEVICE_NAME_USER        "vboxuser"
 /** The name of the PCI driver */
 #define DRIVER_NAME             DEVICE_NAME
@@ -71,19 +71,20 @@
 /*********************************************************************************************************************************
 *   Internal Functions                                                                                                           *
 *********************************************************************************************************************************/
-static void vboxguestLinuxTermPci(struct pci_dev *pPciDev);
-static int  vboxguestLinuxModInit(void);
-static void vboxguestLinuxModExit(void);
-static int  vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp);
-static int  vboxguestLinuxRelease(struct inode *pInode, struct file *pFilp);
+static void vgdrvLinuxTermPci(struct pci_dev *pPciDev);
+static int  vgdrvLinuxProbePci(struct pci_dev *pPciDev, const struct pci_device_id *id);
+static int  vgdrvLinuxModInit(void);
+static void vgdrvLinuxModExit(void);
+static int  vgdrvLinuxOpen(struct inode *pInode, struct file *pFilp);
+static int  vgdrvLinuxRelease(struct inode *pInode, struct file *pFilp);
 #ifdef HAVE_UNLOCKED_IOCTL
-static long vboxguestLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+static long vgdrvLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
 #else
-static int  vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
+static int  vgdrvLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg);
 #endif
-static int  vboxguestFAsync(int fd, struct file *pFile, int fOn);
-static unsigned int vboxguestPoll(struct file *pFile, poll_table *pPt);
-static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff);
+static int  vgdrvLinuxFAsync(int fd, struct file *pFile, int fOn);
+static unsigned int vgdrvLinuxPoll(struct file *pFile, poll_table *pPt);
+static ssize_t vgdrvLinuxRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff);
 
 
 /*********************************************************************************************************************************
@@ -140,16 +141,16 @@ static struct input_dev        *g_pInputDevice = NULL;
 static struct file_operations   g_FileOps =
 {
     owner:          THIS_MODULE,
-    open:           vboxguestLinuxOpen,
-    release:        vboxguestLinuxRelease,
+    open:           vgdrvLinuxOpen,
+    release:        vgdrvLinuxRelease,
 #ifdef HAVE_UNLOCKED_IOCTL
-    unlocked_ioctl: vboxguestLinuxIOCtl,
+    unlocked_ioctl: vgdrvLinuxIOCtl,
 #else
-    ioctl:          vboxguestLinuxIOCtl,
+    ioctl:          vgdrvLinuxIOCtl,
 #endif
-    fasync:         vboxguestFAsync,
-    read:           vboxguestRead,
-    poll:           vboxguestPoll,
+    fasync:         vgdrvLinuxFAsync,
+    read:           vgdrvLinuxRead,
+    poll:           vgdrvLinuxPoll,
     llseek:         no_llseek,
 };
 
@@ -167,12 +168,12 @@ static struct miscdevice        g_MiscDevice =
 static struct file_operations   g_FileOpsUser =
 {
     owner:          THIS_MODULE,
-    open:           vboxguestLinuxOpen,
-    release:        vboxguestLinuxRelease,
+    open:           vgdrvLinuxOpen,
+    release:        vgdrvLinuxRelease,
 #ifdef HAVE_UNLOCKED_IOCTL
-    unlocked_ioctl: vboxguestLinuxIOCtl,
+    unlocked_ioctl: vgdrvLinuxIOCtl,
 #else
-    ioctl:          vboxguestLinuxIOCtl,
+    ioctl:          vgdrvLinuxIOCtl,
 #endif
 };
 
@@ -200,17 +201,29 @@ g_VBoxGuestPciId[] =
         /* empty entry */
     }
 };
+
 MODULE_DEVICE_TABLE(pci, g_VBoxGuestPciId);
 
+/** Structure for registering the PCI driver. */
+static struct pci_driver  g_PciDriver =
+{
+    name:           DRIVER_NAME,
+    id_table:       g_VBoxGuestPciId,
+    probe:          vgdrvLinuxProbePci,
+    remove:         vgdrvLinuxTermPci
+};
+
 static PVBOXGUESTSESSION        g_pKernelSession = NULL;
 
+
+
 /**
  * Converts a VBox status code to a linux error code.
  *
  * @returns corresponding negative linux error code.
  * @param   rc  supdrv error code (SUPDRV_ERR_* defines).
  */
-static int vboxguestLinuxConvertToNegErrno(int rc)
+static int vgdrvLinuxConvertToNegErrno(int rc)
 {
     if (   rc > -1000
         && rc < 1000)
@@ -233,16 +246,14 @@ static int vboxguestLinuxConvertToNegErrno(int rc)
 }
 
 
-
 /**
  * Does the PCI detection and init of the device.
  *
  * @returns 0 on success, negated errno on failure.
  */
-static int vboxguestLinuxProbePci(struct pci_dev *pPciDev,
-                                  const struct pci_device_id *id)
+static int vgdrvLinuxProbePci(struct pci_dev *pPciDev, const struct pci_device_id *id)
 {
-    int             rc;
+    int rc;
 
     NOREF(id);
     AssertReturn(!g_pPciDev, -EINVAL);
@@ -298,7 +309,7 @@ static int vboxguestLinuxProbePci(struct pci_dev *pPciDev,
 /**
  * Clean up the usage of the PCI device.
  */
-static void vboxguestLinuxTermPci(struct pci_dev *pPciDev)
+static void vgdrvLinuxTermPci(struct pci_dev *pPciDev)
 {
     g_pPciDev = NULL;
     if (pPciDev)
@@ -315,16 +326,6 @@ static void vboxguestLinuxTermPci(struct pci_dev *pPciDev)
 }
 
 
-/** Structure for registering the PCI driver. */
-static struct pci_driver  g_PciDriver =
-{
-    name:           DRIVER_NAME,
-    id_table:       g_VBoxGuestPciId,
-    probe:          vboxguestLinuxProbePci,
-    remove:         vboxguestLinuxTermPci
-};
-
-
 /**
  * Interrupt service routine.
  *
@@ -333,15 +334,15 @@ static struct pci_driver  g_PciDriver =
  *
  * @param   iIrq            The IRQ number.
  * @param   pvDevId         The device ID, a pointer to g_DevExt.
- * @param   pvRegs          Register set. Removed in 2.6.19.
+ * @param   pRegs           Register set. Removed in 2.6.19.
  */
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
-static irqreturn_t vboxguestLinuxISR(int iIrrq, void *pvDevId)
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19) && !defined(DOXYGEN_RUNNING)
+static irqreturn_t vgdrvLinuxISR(int iIrq, void *pvDevId)
 #else
-static irqreturn_t vboxguestLinuxISR(int iIrrq, void *pvDevId, struct pt_regs *pRegs)
+static irqreturn_t vgdrvLinuxISR(int iIrq, void *pvDevId, struct pt_regs *pRegs)
 #endif
 {
-    bool fTaken = VbgdCommonISR(&g_DevExt);
+    bool fTaken = VGDrvCommonISR(&g_DevExt);
     return IRQ_RETVAL(fTaken);
 }
 
@@ -349,13 +350,13 @@ static irqreturn_t vboxguestLinuxISR(int iIrrq, void *pvDevId, struct pt_regs *p
 /**
  * Registers the ISR and initializes the poll wait queue.
  */
-static int __init vboxguestLinuxInitISR(void)
+static int __init vgdrvLinuxInitISR(void)
 {
     int rc;
 
     init_waitqueue_head(&g_PollEventQueue);
     rc = request_irq(g_pPciDev->irq,
-                     vboxguestLinuxISR,
+                     vgdrvLinuxISR,
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 20)
                      IRQF_SHARED,
 #else
@@ -375,110 +376,120 @@ static int __init vboxguestLinuxInitISR(void)
 /**
  * Deregisters the ISR.
  */
-static void vboxguestLinuxTermISR(void)
+static void vgdrvLinuxTermISR(void)
 {
     free_irq(g_pPciDev->irq, &g_DevExt);
 }
 
 
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-/** Calls the kernel IOCtl to report mouse status to the host on behalf of
- * our kernel session. */
-static int vboxguestLinuxSetMouseStatus(uint32_t fStatus)
+
+/**
+ * Reports the mouse integration status to the host.
+ *
+ * Calls the kernel IOCtl to report mouse status to the host on behalf of
+ * our kernel session.
+ *
+ * @param   fStatus     The mouse status to report.
+ */
+static int vgdrvLinuxSetMouseStatus(uint32_t fStatus)
 {
-    return VbgdCommonIoCtl(VBOXGUEST_IOCTL_SET_MOUSE_STATUS, &g_DevExt,
-                           g_pKernelSession, &fStatus, sizeof(fStatus),
-                           NULL);
+    return VGDrvCommonIoCtl(VBOXGUEST_IOCTL_SET_MOUSE_STATUS, &g_DevExt, g_pKernelSession, &fStatus, sizeof(fStatus), NULL);
 }
 
 
-/** Called when the input device is first opened.  Sets up absolute reporting.
+/**
+ * Called when the input device is first opened.
+ *
+ * Sets up absolute mouse reporting.
  */
 static int vboxguestOpenInputDevice(struct input_dev *pDev)
 {
-    NOREF(pDev);
-    if (RT_FAILURE(vboxguestLinuxSetMouseStatus
-                                   (  VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE
-                                    | VMMDEV_MOUSE_NEW_PROTOCOL)))
+    int rc = vgdrvLinuxSetMouseStatus(VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE | VMMDEV_MOUSE_NEW_PROTOCOL);
+    if (RT_FAILURE(rc))
         return ENODEV;
+    NOREF(pDev);
     return 0;
 }
 
 
-/** Called if all open handles to the device are closed, disables absolute
- * reporting. */
+/**
+ * Called if all open handles to the input device are closed.
+ *
+ * Disables absolute reporting.
+ */
 static void vboxguestCloseInputDevice(struct input_dev *pDev)
 {
     NOREF(pDev);
-    vboxguestLinuxSetMouseStatus(0);
+    vgdrvLinuxSetMouseStatus(0);
 }
 
 
 /**
  * Creates the kernel input device.
  */
-static int __init vboxguestLinuxCreateInputDevice(void)
+static int __init vgdrvLinuxCreateInputDevice(void)
 {
-    int rc;
-
-    rc = VbglGRAlloc((VMMDevRequestHeader **)&g_pMouseStatusReq,
-                     sizeof(*g_pMouseStatusReq),
-                     VMMDevReq_GetMouseStatus);
-    if (RT_FAILURE(rc))
-        return -ENOMEM;
-    g_pInputDevice = input_allocate_device();
-    if (!g_pInputDevice)
+    int rc = VbglGRAlloc((VMMDevRequestHeader **)&g_pMouseStatusReq, sizeof(*g_pMouseStatusReq), VMMDevReq_GetMouseStatus);
+    if (RT_SUCCESS(rc))
     {
-        VbglGRFree(&g_pMouseStatusReq->header);
-        return -ENOMEM;
-    }
-    g_pInputDevice->id.bustype = BUS_PCI;
-    g_pInputDevice->id.vendor  = VMMDEV_VENDORID;
-    g_pInputDevice->id.product = VMMDEV_DEVICEID;
-    g_pInputDevice->id.version = VBOX_SHORT_VERSION;
-    g_pInputDevice->open       = vboxguestOpenInputDevice;
-    g_pInputDevice->close      = vboxguestCloseInputDevice;
-# if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
-    g_pInputDevice->cdev.dev   = &g_pPciDev->dev;
-# else
-    g_pInputDevice->dev.parent = &g_pPciDev->dev;
-# endif
-    {
-        int rc = input_register_device(g_pInputDevice);
-        if (rc)
+        g_pInputDevice = input_allocate_device();
+        if (g_pInputDevice)
         {
-            VbglGRFree(&g_pMouseStatusReq->header);
-            input_free_device(g_pInputDevice);
-            return rc;
-        }
-    }
-    /* Do what one of our competitors apparently does as that works. */
-    ASMBitSet(g_pInputDevice->evbit, EV_ABS);
-    ASMBitSet(g_pInputDevice->evbit, EV_KEY);
-# ifdef EV_SYN
-    ASMBitSet(g_pInputDevice->evbit, EV_SYN);
+            g_pInputDevice->id.bustype = BUS_PCI;
+            g_pInputDevice->id.vendor  = VMMDEV_VENDORID;
+            g_pInputDevice->id.product = VMMDEV_DEVICEID;
+            g_pInputDevice->id.version = VBOX_SHORT_VERSION;
+            g_pInputDevice->open       = vboxguestOpenInputDevice;
+            g_pInputDevice->close      = vboxguestCloseInputDevice;
+# if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22)
+            g_pInputDevice->cdev.dev   = &g_pPciDev->dev;
+# else
+            g_pInputDevice->dev.parent = &g_pPciDev->dev;
 # endif
-    input_set_abs_params(g_pInputDevice, ABS_X, VMMDEV_MOUSE_RANGE_MIN,
-                         VMMDEV_MOUSE_RANGE_MAX, 0, 0);
-    input_set_abs_params(g_pInputDevice, ABS_Y, VMMDEV_MOUSE_RANGE_MIN,
-                         VMMDEV_MOUSE_RANGE_MAX, 0, 0);
-    ASMBitSet(g_pInputDevice->keybit, BTN_MOUSE);
-    /** @todo this string should be in a header file somewhere. */
-    g_pInputDevice->name = "VirtualBox mouse integration";
-    return 0;
+            rc = input_register_device(g_pInputDevice);
+            if (rc == 0)
+            {
+                /* Do what one of our competitors apparently does as that works. */
+                ASMBitSet(g_pInputDevice->evbit, EV_ABS);
+                ASMBitSet(g_pInputDevice->evbit, EV_KEY);
+# ifdef EV_SYN
+                ASMBitSet(g_pInputDevice->evbit, EV_SYN);
+# endif
+                input_set_abs_params(g_pInputDevice, ABS_X, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
+                input_set_abs_params(g_pInputDevice, ABS_Y, VMMDEV_MOUSE_RANGE_MIN, VMMDEV_MOUSE_RANGE_MAX, 0, 0);
+                ASMBitSet(g_pInputDevice->keybit, BTN_MOUSE);
+                /** @todo this string should be in a header file somewhere. */
+                g_pInputDevice->name = "VirtualBox mouse integration";
+                return 0;
+            }
+
+            input_free_device(g_pInputDevice);
+        }
+        else
+            rc = -ENOMEM;
+        VbglGRFree(&g_pMouseStatusReq->header);
+        g_pMouseStatusReq = NULL;
+    }
+    else
+        rc = -ENOMEM;
+    return rc;
 }
 
 
 /**
  * Terminates the kernel input device.
  */
-static void vboxguestLinuxTermInputDevice(void)
+static void vgdrvLinuxTermInputDevice(void)
 {
     VbglGRFree(&g_pMouseStatusReq->header);
+    g_pMouseStatusReq = NULL;
+
     /* See documentation of input_register_device(): input_free_device()
      * should not be called after a device has been registered. */
     input_unregister_device(g_pInputDevice);
 }
+
 #endif /* VBOXGUEST_WITH_INPUT_DRIVER */
 
 
@@ -487,39 +498,33 @@ static void vboxguestLinuxTermInputDevice(void)
  *
  * @returns 0 on success, negated errno on failure.
  */
-static int __init vboxguestLinuxInitDeviceNodes(void)
+static int __init vgdrvLinuxInitDeviceNodes(void)
 {
-    int rc;
-
     /*
      * The full feature device node.
      */
-    rc = misc_register(&g_MiscDevice);
-    if (rc)
-    {
-        LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME, rc));
-        return rc;
-    }
-
-    /*
-     * The device node intended to be accessible by all users.
-     */
-    rc = misc_register(&g_MiscDeviceUser);
-    if (rc)
+    int rc = misc_register(&g_MiscDevice);
+    if (!rc)
     {
+        /*
+         * The device node intended to be accessible by all users.
+         */
+        rc = misc_register(&g_MiscDeviceUser);
+        if (!rc)
+            return 0;
         LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME_USER, rc));
         misc_deregister(&g_MiscDevice);
-        return rc;
     }
-
-    return 0;
+    else
+        LogRel((DEVICE_NAME ": misc_register failed for %s (rc=%d)\n", DEVICE_NAME, rc));
+    return rc;
 }
 
 
 /**
  * Deregisters the device nodes.
  */
-static void vboxguestLinuxTermDeviceNodes(void)
+static void vgdrvLinuxTermDeviceNodes(void)
 {
     misc_deregister(&g_MiscDevice);
     misc_deregister(&g_MiscDeviceUser);
@@ -531,7 +536,7 @@ static void vboxguestLinuxTermDeviceNodes(void)
  *
  * @returns appropriate status code.
  */
-static int __init vboxguestLinuxModInit(void)
+static int __init vgdrvLinuxModInit(void)
 {
     static const char * const   s_apszGroups[] = VBOX_LOGGROUP_NAMES;
     PRTLOGGER                   pRelLogger;
@@ -577,7 +582,7 @@ static int __init vboxguestLinuxModInit(void)
         /*
          * Register the interrupt service routine for it.
          */
-        rc = vboxguestLinuxInitISR();
+        rc = vgdrvLinuxInitISR();
         if (rc >= 0)
         {
             /*
@@ -595,33 +600,32 @@ static int __init vboxguestLinuxModInit(void)
 # warning "huh? which arch + version is this?"
             VBOXOSTYPE enmOsType = VBOXOSTYPE_Linux;
 #endif
-            rc = VbgdCommonInitDevExt(&g_DevExt,
-                                      g_IOPortBase,
-                                      g_pvMMIOBase,
-                                      g_cbMMIO,
-                                      enmOSType,
-                                      VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
+            rc = VGDrvCommonInitDevExt(&g_DevExt,
+                                       g_IOPortBase,
+                                       g_pvMMIOBase,
+                                       g_cbMMIO,
+                                       enmOSType,
+                                       VMMDEV_EVENT_MOUSE_POSITION_CHANGED);
             if (RT_SUCCESS(rc))
             {
                 /*
                  * Create the kernel session for this driver.
                  */
-                rc = VbgdCommonCreateKernelSession(&g_DevExt,
-                                                  &g_pKernelSession);
+                rc = VGDrvCommonCreateKernelSession(&g_DevExt, &g_pKernelSession);
                 if (RT_SUCCESS(rc))
                 {
                     /*
                      * Create the kernel input device.
                      */
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-                    rc = vboxguestLinuxCreateInputDevice();
+                    rc = vgdrvLinuxCreateInputDevice();
                     if (rc >= 0)
                     {
 #endif
                         /*
                          * Finally, create the device nodes.
                          */
-                        rc = vboxguestLinuxInitDeviceNodes();
+                        rc = vgdrvLinuxInitDeviceNodes();
                         if (rc >= 0)
                         {
                             /* some useful information for the user but don't show this on the console */
@@ -634,7 +638,7 @@ static int __init vboxguestLinuxModInit(void)
 
                         /* bail out */
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-                        vboxguestLinuxTermInputDevice();
+                        vgdrvLinuxTermInputDevice();
                     }
                     else
                     {
@@ -642,16 +646,16 @@ static int __init vboxguestLinuxModInit(void)
                         rc = RTErrConvertFromErrno(rc);
                     }
 #endif
-                    VbgdCommonCloseSession(&g_DevExt, g_pKernelSession);
+                    VGDrvCommonCloseSession(&g_DevExt, g_pKernelSession);
                 }
-                VbgdCommonDeleteDevExt(&g_DevExt);
+                VGDrvCommonDeleteDevExt(&g_DevExt);
             }
             else
             {
-                LogRel((DEVICE_NAME ": VbgdCommonInitDevExt failed with rc=%Rrc\n", rc));
+                LogRel((DEVICE_NAME ": VGDrvCommonInitDevExt failed with rc=%Rrc\n", rc));
                 rc = RTErrConvertFromErrno(rc);
             }
-            vboxguestLinuxTermISR();
+            vgdrvLinuxTermISR();
         }
     }
     else
@@ -670,18 +674,18 @@ static int __init vboxguestLinuxModInit(void)
 /**
  * Unload the module.
  */
-static void __exit vboxguestLinuxModExit(void)
+static void __exit vgdrvLinuxModExit(void)
 {
     /*
      * Inverse order of init.
      */
-    vboxguestLinuxTermDeviceNodes();
+    vgdrvLinuxTermDeviceNodes();
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
-    vboxguestLinuxTermInputDevice();
+    vgdrvLinuxTermInputDevice();
 #endif
-    VbgdCommonCloseSession(&g_DevExt, g_pKernelSession);
-    VbgdCommonDeleteDevExt(&g_DevExt);
-    vboxguestLinuxTermISR();
+    VGDrvCommonCloseSession(&g_DevExt, g_pKernelSession);
+    VGDrvCommonDeleteDevExt(&g_DevExt);
+    vgdrvLinuxTermISR();
     pci_unregister_driver(&g_PciDriver);
     RTLogDestroy(RTLogRelSetDefaultInstance(NULL));
     RTLogDestroy(RTLogSetDefaultInstance(NULL));
@@ -695,7 +699,7 @@ static void __exit vboxguestLinuxModExit(void)
  * @param   pInode      Pointer to inode info structure.
  * @param   pFilp       Associated file pointer.
  */
-static int vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp)
+static int vgdrvLinuxOpen(struct inode *pInode, struct file *pFilp)
 {
     int                 rc;
     PVBOXGUESTSESSION   pSession;
@@ -705,7 +709,7 @@ static int vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp)
      * Call common code to create the user session. Associate it with
      * the file so we can access it in the other methods.
      */
-    rc = VbgdCommonCreateUserSession(&g_DevExt, &pSession);
+    rc = VGDrvCommonCreateUserSession(&g_DevExt, &pSession);
     if (RT_SUCCESS(rc))
     {
         pFilp->private_data = pSession;
@@ -713,10 +717,9 @@ static int vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp)
             pSession->fUserSession = true;
     }
 
-    Log(("vboxguestLinuxOpen: g_DevExt=%p pSession=%p rc=%d/%d (pid=%d/%d %s)\n",
-         &g_DevExt, pSession, rc, vboxguestLinuxConvertToNegErrno(rc),
-         RTProcSelf(), current->pid, current->comm));
-    return vboxguestLinuxConvertToNegErrno(rc);
+    Log(("vgdrvLinuxOpen: g_DevExt=%p pSession=%p rc=%d/%d (pid=%d/%d %s)\n",
+         &g_DevExt, pSession, rc, vgdrvLinuxConvertToNegErrno(rc), RTProcSelf(), current->pid, current->comm));
+    return vgdrvLinuxConvertToNegErrno(rc);
 }
 
 
@@ -726,17 +729,17 @@ static int vboxguestLinuxOpen(struct inode *pInode, struct file *pFilp)
  * @param   pInode      Pointer to inode info structure.
  * @param   pFilp       Associated file pointer.
  */
-static int vboxguestLinuxRelease(struct inode *pInode, struct file *pFilp)
+static int vgdrvLinuxRelease(struct inode *pInode, struct file *pFilp)
 {
-    Log(("vboxguestLinuxRelease: pFilp=%p pSession=%p pid=%d/%d %s\n",
+    Log(("vgdrvLinuxRelease: pFilp=%p pSession=%p pid=%d/%d %s\n",
          pFilp, pFilp->private_data, RTProcSelf(), current->pid, current->comm));
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
     /* This housekeeping was needed in older kernel versions to ensure that
      * the file pointer didn't get left on the polling queue. */
-    vboxguestFAsync(-1, pFilp, 0);
+    vgdrvLinuxFAsync(-1, pFilp, 0);
 #endif
-    VbgdCommonCloseSession(&g_DevExt, (PVBOXGUESTSESSION)pFilp->private_data);
+    VGDrvCommonCloseSession(&g_DevExt, (PVBOXGUESTSESSION)pFilp->private_data);
     pFilp->private_data = NULL;
     return 0;
 }
@@ -745,14 +748,15 @@ static int vboxguestLinuxRelease(struct inode *pInode, struct file *pFilp)
 /**
  * Device I/O Control entry point.
  *
+ * @param   pInode      Associated inode pointer.
  * @param   pFilp       Associated file pointer.
  * @param   uCmd        The function specified to ioctl().
  * @param   ulArg       The argument specified to ioctl().
  */
 #ifdef HAVE_UNLOCKED_IOCTL
-static long vboxguestLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
+static long vgdrvLinuxIOCtl(struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
 #else
-static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
+static int vgdrvLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigned int uCmd, unsigned long ulArg)
 #endif
 {
     PVBOXGUESTSESSION   pSession = (PVBOXGUESTSESSION)pFilp->private_data;
@@ -762,7 +766,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
     int                 rc;
     uint64_t            au64Buf[32/sizeof(uint64_t)];
 
-    Log6(("vboxguestLinuxIOCtl: pFilp=%p uCmd=%#x ulArg=%p pid=%d/%d\n", pFilp, uCmd, (void *)ulArg, RTProcSelf(), current->pid));
+    Log6(("vgdrvLinuxIOCtl: pFilp=%p uCmd=%#x ulArg=%p pid=%d/%d\n", pFilp, uCmd, (void *)ulArg, RTProcSelf(), current->pid));
 
     /*
      * Buffer the request.
@@ -787,7 +791,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
          * Process the IOCtl.
          */
         size_t cbDataReturned;
-        rc = VbgdCommonIoCtl(uCmd, &g_DevExt, pSession, pvBuf, cbData, &cbDataReturned);
+        rc = VGDrvCommonIoCtl(uCmd, &g_DevExt, pSession, pvBuf, cbData, &cbDataReturned);
 
         /*
          * Copy ioctl data and output buffer back to user space.
@@ -812,7 +816,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
         }
         else
         {
-            Log(("vboxguestLinuxIOCtl: pFilp=%p uCmd=%#x ulArg=%p failed, rc=%d\n", pFilp, uCmd, (void *)ulArg, rc));
+            Log(("vgdrvLinuxIOCtl: pFilp=%p uCmd=%#x ulArg=%p failed, rc=%d\n", pFilp, uCmd, (void *)ulArg, rc));
             rc = -rc; Assert(rc > 0); /* Positive returns == negated VBox error status codes. */
         }
     }
@@ -824,7 +828,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
     if (pvBufFree)
         RTMemFree(pvBufFree);
 
-    Log6(("vboxguestLinuxIOCtl: returns %d (pid=%d/%d)\n", rc, RTProcSelf(), current->pid));
+    Log6(("vgdrvLinuxIOCtl: returns %d (pid=%d/%d)\n", rc, RTProcSelf(), current->pid));
     return rc;
 }
 
@@ -838,7 +842,7 @@ static int vboxguestLinuxIOCtl(struct inode *pInode, struct file *pFilp, unsigne
  * @param   pFile       The file structure.
  * @param   fOn         On/off indicator.
  */
-static int vboxguestFAsync(int fd, struct file *pFile, int fOn)
+static int vgdrvLinuxFAsync(int fd, struct file *pFile, int fOn)
 {
     return fasync_helper(fd, pFile, fOn, &g_pFAsyncQueue);
 }
@@ -858,7 +862,7 @@ static int vboxguestFAsync(int fd, struct file *pFile, int fOn)
  * @remarks This is probably not really used, X11 is said to use the fasync
  *          interface instead.
  */
-static unsigned int vboxguestPoll(struct file *pFile, poll_table *pPt)
+static unsigned int vgdrvLinuxPoll(struct file *pFile, poll_table *pPt)
 {
     PVBOXGUESTSESSION   pSession  = (PVBOXGUESTSESSION)pFile->private_data;
     uint32_t            u32CurSeq = ASMAtomicUoReadU32(&g_DevExt.u32MousePosChangedSeq);
@@ -882,9 +886,9 @@ static unsigned int vboxguestPoll(struct file *pFile, poll_table *pPt)
  *
  * @remarks This is probably not really used as X11 lets the driver do its own
  *          event reading. The poll condition is therefore also cleared when we
- *          see VMMDevReq_GetMouseStatus in VbgdCommonIoCtl_VMMRequest.
+ *          see VMMDevReq_GetMouseStatus in vgdrvIoCtl_VMMRequest.
  */
-static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff)
+static ssize_t vgdrvLinuxRead(struct file *pFile, char *pbBuf, size_t cbRead, loff_t *poff)
 {
     PVBOXGUESTSESSION   pSession  = (PVBOXGUESTSESSION)pFile->private_data;
     uint32_t            u32CurSeq = ASMAtomicUoReadU32(&g_DevExt.u32MousePosChangedSeq);
@@ -906,7 +910,7 @@ static ssize_t vboxguestRead(struct file *pFile, char *pbBuf, size_t cbRead, lof
 }
 
 
-void VbgdNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
+void VGDrvNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 {
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     int rc;
@@ -917,9 +921,9 @@ void VbgdNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
      * Wake up everyone that's in a poll() and post anyone that has
      * subscribed to async notifications.
      */
-    Log3(("VbgdNativeISRMousePollEvent: wake_up_all\n"));
+    Log3(("VGDrvNativeISRMousePollEvent: wake_up_all\n"));
     wake_up_all(&g_PollEventQueue);
-    Log3(("VbgdNativeISRMousePollEvent: kill_fasync\n"));
+    Log3(("VGDrvNativeISRMousePollEvent: kill_fasync\n"));
     kill_fasync(&g_pFAsyncQueue, SIGIO, POLL_IN);
 #ifdef VBOXGUEST_WITH_INPUT_DRIVER
     /* Report events to the kernel input device */
@@ -938,7 +942,7 @@ void VbgdNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
 # endif
     }
 #endif
-    Log3(("VbgdNativeISRMousePollEvent: done\n"));
+    Log3(("VGDrvNativeISRMousePollEvent: done\n"));
 }
 
 
@@ -953,7 +957,7 @@ EXPORT_SYMBOL(VBoxGuestIDCCall);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
 
 /** log and dbg_log parameter setter. */
-static int vboxguestLinuxParamLogGrpSet(const char *pszValue, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogGrpSet(const char *pszValue, struct kernel_param *pParam)
 {
     if (g_fLoggerCreated)
     {
@@ -968,7 +972,7 @@ static int vboxguestLinuxParamLogGrpSet(const char *pszValue, struct kernel_para
 }
 
 /** log and dbg_log parameter getter. */
-static int vboxguestLinuxParamLogGrpGet(char *pszBuf, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogGrpGet(char *pszBuf, struct kernel_param *pParam)
 {
     PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
@@ -979,7 +983,7 @@ static int vboxguestLinuxParamLogGrpGet(char *pszBuf, struct kernel_param *pPara
 
 
 /** log and dbg_log_flags parameter setter. */
-static int vboxguestLinuxParamLogFlagsSet(const char *pszValue, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogFlagsSet(const char *pszValue, struct kernel_param *pParam)
 {
     if (g_fLoggerCreated)
     {
@@ -993,7 +997,7 @@ static int vboxguestLinuxParamLogFlagsSet(const char *pszValue, struct kernel_pa
 }
 
 /** log and dbg_log_flags parameter getter. */
-static int vboxguestLinuxParamLogFlagsGet(char *pszBuf, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogFlagsGet(char *pszBuf, struct kernel_param *pParam)
 {
     PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
@@ -1004,7 +1008,7 @@ static int vboxguestLinuxParamLogFlagsGet(char *pszBuf, struct kernel_param *pPa
 
 
 /** log and dbg_log_dest parameter setter. */
-static int vboxguestLinuxParamLogDstSet(const char *pszValue, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogDstSet(const char *pszValue, struct kernel_param *pParam)
 {
     if (g_fLoggerCreated)
     {
@@ -1018,7 +1022,7 @@ static int vboxguestLinuxParamLogDstSet(const char *pszValue, struct kernel_para
 }
 
 /** log and dbg_log_dest parameter getter. */
-static int vboxguestLinuxParamLogDstGet(char *pszBuf, struct kernel_param *pParam)
+static int vgdrvLinuxParamLogDstGet(char *pszBuf, struct kernel_param *pParam)
 {
     PRTLOGGER pLogger = pParam->name[0] == 'd' ? RTLogDefaultInstance() : RTLogRelGetDefaultInstance();
     *pszBuf = '\0';
@@ -1029,7 +1033,7 @@ static int vboxguestLinuxParamLogDstGet(char *pszBuf, struct kernel_param *pPara
 
 
 /** r3_log_to_host parameter setter. */
-static int vboxguestLinuxParamR3LogToHostSet(const char *pszValue, struct kernel_param *pParam)
+static int vgdrvLinuxParamR3LogToHostSet(const char *pszValue, struct kernel_param *pParam)
 {
     if (    pszValue == NULL
         || *pszValue == '\0'
@@ -1047,7 +1051,7 @@ static int vboxguestLinuxParamR3LogToHostSet(const char *pszValue, struct kernel
 }
 
 /** r3_log_to_host parameter getter. */
-static int vboxguestLinuxParamR3LogToHostGet(char *pszBuf, struct kernel_param *pParam)
+static int vgdrvLinuxParamR3LogToHostGet(char *pszBuf, struct kernel_param *pParam)
 {
     strcpy(pszBuf, g_DevExt.fLoggingEnabled ? "enabled" : "disabled");
     return strlen(pszBuf);
@@ -1057,21 +1061,21 @@ static int vboxguestLinuxParamR3LogToHostGet(char *pszBuf, struct kernel_param *
 /*
  * Define module parameters.
  */
-module_param_call(log,            vboxguestLinuxParamLogGrpSet,   vboxguestLinuxParamLogGrpGet,   NULL, 0664);
-module_param_call(log_flags,      vboxguestLinuxParamLogFlagsSet, vboxguestLinuxParamLogFlagsGet, NULL, 0664);
-module_param_call(log_dest,       vboxguestLinuxParamLogDstSet,   vboxguestLinuxParamLogDstGet,   NULL, 0664);
+module_param_call(log,            vgdrvLinuxParamLogGrpSet,   vgdrvLinuxParamLogGrpGet,   NULL, 0664);
+module_param_call(log_flags,      vgdrvLinuxParamLogFlagsSet, vgdrvLinuxParamLogFlagsGet, NULL, 0664);
+module_param_call(log_dest,       vgdrvLinuxParamLogDstSet,   vgdrvLinuxParamLogDstGet,   NULL, 0664);
 # ifdef LOG_ENABLED
-module_param_call(dbg_log,        vboxguestLinuxParamLogGrpSet,   vboxguestLinuxParamLogGrpGet,   NULL, 0664);
-module_param_call(dbg_log_flags,  vboxguestLinuxParamLogFlagsSet, vboxguestLinuxParamLogFlagsGet, NULL, 0664);
-module_param_call(dbg_log_dest,   vboxguestLinuxParamLogDstSet,   vboxguestLinuxParamLogDstGet,   NULL, 0664);
+module_param_call(dbg_log,        vgdrvLinuxParamLogGrpSet,   vgdrvLinuxParamLogGrpGet,   NULL, 0664);
+module_param_call(dbg_log_flags,  vgdrvLinuxParamLogFlagsSet, vgdrvLinuxParamLogFlagsGet, NULL, 0664);
+module_param_call(dbg_log_dest,   vgdrvLinuxParamLogDstSet,   vgdrvLinuxParamLogDstGet,   NULL, 0664);
 # endif
-module_param_call(r3_log_to_host, vboxguestLinuxParamR3LogToHostSet, vboxguestLinuxParamR3LogToHostGet, NULL, 0664);
+module_param_call(r3_log_to_host, vgdrvLinuxParamR3LogToHostSet, vgdrvLinuxParamR3LogToHostGet, NULL, 0664);
 
 #endif /* 2.6.0 and later */
 
 
-module_init(vboxguestLinuxModInit);
-module_exit(vboxguestLinuxModExit);
+module_init(vgdrvLinuxModInit);
+module_exit(vgdrvLinuxModExit);
 
 MODULE_AUTHOR(VBOX_VENDOR);
 MODULE_DESCRIPTION(VBOX_PRODUCT " Guest Additions for Linux Module");
