@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2010-2015 Oracle Corporation
+ * Copyright (C) 2010-2016 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -65,41 +65,39 @@ typedef struct DRVHOSTCOREAUDIO
  *
  ******************************************************************************/
 
-#ifdef DEBUG
 static void drvHostCoreAudioPrintASBDesc(const char *pszDesc, const AudioStreamBasicDescription *pStreamDesc)
 {
     char pszSampleRate[32];
-    Log(("%s AudioStreamBasicDescription:\n", pszDesc));
-    LogFlowFunc(("Format ID: %RU32 (%c%c%c%c)\n", pStreamDesc->mFormatID,
-                 RT_BYTE4(pStreamDesc->mFormatID), RT_BYTE3(pStreamDesc->mFormatID),
-                 RT_BYTE2(pStreamDesc->mFormatID), RT_BYTE1(pStreamDesc->mFormatID)));
-    LogFlowFunc(("Flags: %RU32", pStreamDesc->mFormatFlags));
+    LogRel2(("CoreAudio: %s description:\n", pszDesc));
+    LogRel2(("CoreAudio: Format ID: %RU32 (%c%c%c%c)\n", pStreamDesc->mFormatID,
+             RT_BYTE4(pStreamDesc->mFormatID), RT_BYTE3(pStreamDesc->mFormatID),
+             RT_BYTE2(pStreamDesc->mFormatID), RT_BYTE1(pStreamDesc->mFormatID)));
+    LogRel2(("CoreAudio: Flags: %RU32", pStreamDesc->mFormatFlags));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsFloat)
-        Log((" Float"));
+        LogRel2((" Float"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsBigEndian)
-        Log((" BigEndian"));
+        LogRel2((" BigEndian"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsSignedInteger)
-        Log((" SignedInteger"));
+        LogRel2((" SignedInteger"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsPacked)
-        Log((" Packed"));
+        LogRel2((" Packed"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsAlignedHigh)
-        Log((" AlignedHigh"));
+        LogRel2((" AlignedHigh"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsNonInterleaved)
-        Log((" NonInterleaved"));
+        LogRel2((" NonInterleaved"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagIsNonMixable)
-        Log((" NonMixable"));
+        LogRel2((" NonMixable"));
     if (pStreamDesc->mFormatFlags & kAudioFormatFlagsAreAllClear)
-        Log((" AllClear"));
-    Log(("\n"));
+        LogRel2((" AllClear"));
+    LogRel2(("\n"));
     snprintf(pszSampleRate, 32, "%.2f", (float)pStreamDesc->mSampleRate); /** @todo r=andy Use RTStrPrint*. */
-    LogFlowFunc(("SampleRate      : %s\n", pszSampleRate));
-    LogFlowFunc(("ChannelsPerFrame: %RU32\n", pStreamDesc->mChannelsPerFrame));
-    LogFlowFunc(("FramesPerPacket : %RU32\n", pStreamDesc->mFramesPerPacket));
-    LogFlowFunc(("BitsPerChannel  : %RU32\n", pStreamDesc->mBitsPerChannel));
-    LogFlowFunc(("BytesPerFrame   : %RU32\n", pStreamDesc->mBytesPerFrame));
-    LogFlowFunc(("BytesPerPacket  : %RU32\n", pStreamDesc->mBytesPerPacket));
+    LogRel2(("CoreAudio: SampleRate      : %s\n", pszSampleRate));
+    LogRel2(("CoreAudio: ChannelsPerFrame: %RU32\n", pStreamDesc->mChannelsPerFrame));
+    LogRel2(("CoreAudio: FramesPerPacket : %RU32\n", pStreamDesc->mFramesPerPacket));
+    LogRel2(("CoreAudio: BitsPerChannel  : %RU32\n", pStreamDesc->mBitsPerChannel));
+    LogRel2(("CoreAudio: BytesPerFrame   : %RU32\n", pStreamDesc->mBytesPerFrame));
+    LogRel2(("CoreAudio: BytesPerPacket  : %RU32\n", pStreamDesc->mBytesPerPacket));
 }
-#endif /* DEBUG */
 
 static void drvHostCoreAudioPCMInfoToASBDesc(PDMPCMPROPS *pPcmProperties, AudioStreamBasicDescription *pStreamDesc)
 {
@@ -266,18 +264,18 @@ typedef struct COREAUDIOSTREAMOUT
 {
     /** Host stream out. */
     PDMAUDIOHSTSTRMOUT          streamOut;
-    /* Stream description which is default on the device */
+    /** Stream description which is default on the device. */
     AudioStreamBasicDescription deviceFormat;
-    /* Stream description which is selected for using by VBox */
+    /** Stream description which is selected for using with VBox. */
     AudioStreamBasicDescription streamFormat;
-    /* The audio device ID of the currently used device */
+    /** The audio device ID of the currently used device. */
     AudioDeviceID               deviceID;
-    /* The AudioUnit used */
+    /** The AudioUnit being used. */
     AudioUnit                   audioUnit;
-    /* A ring buffer for transferring data to the playback thread. */
+    /** A ring buffer for transferring data to the playback thread. */
     PRTCIRCBUF                  pBuf;
-    /* Initialization status tracker. Used when some of the device parameters
-     * or the device itself is changed during the runtime. */
+    /** Initialization status tracker. Used when some of the device parameters
+     *  or the device itself is changed during the runtime. */
     volatile uint32_t           status;
     /** Flag whether the "default device changed" listener was registered. */
     bool                        fDefDevChgListReg;
@@ -287,26 +285,26 @@ typedef struct COREAUDIOSTREAMIN
 {
     /** Host stream in. */
     PDMAUDIOHSTSTRMIN           streamIn;
-    /* Stream description which is default on the device */
+    /** Stream description which is default on the device. */
     AudioStreamBasicDescription deviceFormat;
-    /* Stream description which is selected for using by VBox */
+    /** Stream description which is selected for using with VBox. */
     AudioStreamBasicDescription streamFormat;
-    /* The audio device ID of the currently used device */
+    /** The audio device ID of the currently used device. */
     AudioDeviceID               deviceID;
-    /* The AudioUnit used */
+    /** The AudioUnit used. */
     AudioUnit                   audioUnit;
-    /* The audio converter if necessary */
-    AudioConverterRef           converter;
-    /* A temporary position value used in the caConverterCallback function */
-    uint32_t                    rpos;
-    /* The ratio between the device & the stream sample rate */
-    Float64                     sampleRatio;
-    /* An extra buffer used for render the audio data in the recording thread */
+    /** The audio converter if necessary. */
+    AudioConverterRef           pConverter;
+    /** Native buffer used for render the audio data in the recording thread. */
     AudioBufferList             bufferList;
-    /* A ring buffer for transferring data from the recording thread */
+    /** Reading offset for the bufferList's buffer. */
+    uint32_t                    offBufferRead;
+    /** The ratio between the device & the stream sample rate. */
+    Float64                     sampleRatio;
+    /** A ring buffer for transferring data from the recording thread. */
     PRTCIRCBUF                  pBuf;
-    /* Initialization status tracker. Used when some of the device parameters
-     * or the device itself is changed during the runtime. */
+    /** Initialization status tracker. Used when some of the device parameters
+     *  or the device itself is changed during the runtime. */
     volatile uint32_t           status;
     /** Flag whether the "default device changed" listener was registered. */
     bool                        fDefDevChgListReg;
@@ -408,12 +406,18 @@ static int drvHostCoreAudioReinitInput(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTST
 
     PCOREAUDIOSTREAMIN pStreamIn = (PCOREAUDIOSTREAMIN)pHstStrmIn;
 
-    drvHostCoreAudioFiniIn(pInterface, &pStreamIn->streamIn);
+    int rc = drvHostCoreAudioFiniIn(pInterface, &pStreamIn->streamIn);
+    if (RT_SUCCESS(rc))
+    {
+        rc = drvHostCoreAudioInitInput(&pStreamIn->streamIn, NULL /* pcSamples */);
+        if (RT_SUCCESS(rc))
+            rc = drvHostCoreAudioControlIn(pInterface, &pStreamIn->streamIn, PDMAUDIOSTREAMCMD_ENABLE);
+    }
 
-    drvHostCoreAudioInitInput(&pStreamIn->streamIn, NULL /* pcSamples */);
-    drvHostCoreAudioControlIn(pInterface, &pStreamIn->streamIn, PDMAUDIOSTREAMCMD_ENABLE);
+    if (RT_FAILURE(rc))
+        LogRel(("CoreAudio: Unable to re-init input stream: %Rrc\n", rc));
 
-    return VINF_SUCCESS;
+    return rc;
 }
 
 static int drvHostCoreAudioReinitOutput(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTSTRMOUT pHstStrmOut)
@@ -423,12 +427,18 @@ static int drvHostCoreAudioReinitOutput(PPDMIHOSTAUDIO pInterface, PPDMAUDIOHSTS
 
     PCOREAUDIOSTREAMOUT pStreamOut = (PCOREAUDIOSTREAMOUT)pHstStrmOut;
 
-    drvHostCoreAudioFiniOut(pInterface, &pStreamOut->streamOut);
+    int rc = drvHostCoreAudioFiniOut(pInterface, &pStreamOut->streamOut);
+    if (RT_SUCCESS(rc))
+    {
+        rc = drvHostCoreAudioInitOutput(&pStreamOut->streamOut, NULL /* pcSamples */);
+        if (RT_SUCCESS(rc))
+            rc = drvHostCoreAudioControlOut(pInterface, &pStreamOut->streamOut, PDMAUDIOSTREAMCMD_ENABLE);
+    }
 
-    drvHostCoreAudioInitOutput(&pStreamOut->streamOut, NULL /* pcSamples */);
-    drvHostCoreAudioControlOut(pInterface, &pStreamOut->streamOut, PDMAUDIOSTREAMCMD_ENABLE);
+    if (RT_FAILURE(rc))
+        LogRel(("CoreAudio: Unable to re-init output stream: %Rrc\n", rc));
 
-    return VINF_SUCCESS;
+    return rc;
 }
 
 /* Callback for getting notified when some of the properties of an audio device has changed. */
@@ -488,9 +498,9 @@ static DECLCALLBACK(OSStatus) drvHostCoreAudioConverterCallback(AudioConverterRe
      *        so go through all buffers not only the first one like now. */
 
     /* Use the lower one of the packets to process & the available packets in the buffer. */
-    Assert(pBufferList->mBuffers[0].mDataByteSize >= pStreamIn->rpos);
+    Assert(pBufferList->mBuffers[0].mDataByteSize >= pStreamIn->offBufferRead);
     UInt32 cSize = RT_MIN(*pcPackets * pStreamIn->deviceFormat.mBytesPerPacket,
-                          pBufferList->mBuffers[0].mDataByteSize - pStreamIn->rpos);
+                          pBufferList->mBuffers[0].mDataByteSize - pStreamIn->offBufferRead);
 
     /* Set the new size on output, so the caller know what we have processed. */
     Assert(pStreamIn->deviceFormat.mBytesPerPacket);
@@ -511,9 +521,9 @@ static DECLCALLBACK(OSStatus) drvHostCoreAudioConverterCallback(AudioConverterRe
     {
         pBufData->mBuffers[0].mNumberChannels = pBufferList->mBuffers[0].mNumberChannels;
         pBufData->mBuffers[0].mDataByteSize   = cSize;
-        pBufData->mBuffers[0].mData           = (uint8_t *)pBufferList->mBuffers[0].mData + pStreamIn->rpos;
+        pBufData->mBuffers[0].mData           = (uint8_t *)pBufferList->mBuffers[0].mData + pStreamIn->offBufferRead;
 
-        pStreamIn->rpos += cSize;
+        pStreamIn->offBufferRead += cSize;
 
         err = noErr;
     }
@@ -545,7 +555,7 @@ static DECLCALLBACK(OSStatus) drvHostCoreAudioRecordingCallback(void
     do
     {
         /* Are we using a converter? */
-        if (pStreamIn->converter)
+        if (pStreamIn->pConverter)
         {
             /* First, render the data as usual. */
             pStreamIn->bufferList.mBuffers[0].mNumberChannels = pStreamIn->deviceFormat.mChannelsPerFrame;
@@ -573,6 +583,9 @@ static DECLCALLBACK(OSStatus) drvHostCoreAudioRecordingCallback(void
             tmpList.mNumberBuffers = 1;
             tmpList.mBuffers[0].mNumberChannels = pStreamIn->streamFormat.mChannelsPerFrame;
 
+            /* Set the read position to zero. */
+            pStreamIn->offBufferRead = 0;
+
             /* Iterate as long as data is available. */
             uint8_t *puDst = NULL;
             while (cbAvail)
@@ -592,9 +605,9 @@ static DECLCALLBACK(OSStatus) drvHostCoreAudioRecordingCallback(void
                 tmpList.mBuffers[0].mDataByteSize = cbToWrite;
                 tmpList.mBuffers[0].mData         = puDst;
 
-                AudioConverterReset(pStreamIn->converter);
+                AudioConverterReset(pStreamIn->pConverter);
 
-                err = AudioConverterFillComplexBuffer(pStreamIn->converter, drvHostCoreAudioConverterCallback, pStreamIn,
+                err = AudioConverterFillComplexBuffer(pStreamIn->pConverter, drvHostCoreAudioConverterCallback, pStreamIn,
                                                       &ioOutputDataPacketSize, &tmpList, NULL);
                 if(   err != noErr
                    && err != caConverterEOFDErr)
@@ -806,7 +819,7 @@ static int drvHostCoreAudioInitInput(PPDMAUDIOHSTSTRMIN pHstStrmIn, uint32_t *pc
         return VERR_AUDIO_BACKEND_INIT_FAILED;
     }
 
-    /* Switch the I/O mode for input to off. This is important, as this is a pure input stream. */
+    /* Switch the I/O mode for output to off. This is important, as this is a pure input stream. */
     uFlag = 0;
     err = AudioUnitSetProperty(pStreamIn->audioUnit, kAudioOutputUnitProperty_EnableIO, kAudioUnitScope_Output,
                                0, &uFlag, sizeof(uFlag));
@@ -855,21 +868,20 @@ static int drvHostCoreAudioInitInput(PPDMAUDIOHSTSTRMIN pHstStrmIn, uint32_t *pc
     /* Create an AudioStreamBasicDescription based on our required audio settings. */
     drvHostCoreAudioPCMInfoToASBDesc(&pStreamIn->streamIn.Props, &pStreamIn->streamFormat);
 
-#ifdef DEBUG
     drvHostCoreAudioPrintASBDesc("CoreAudio: Input device", &pStreamIn->deviceFormat);
     drvHostCoreAudioPrintASBDesc("CoreAudio: Input stream", &pStreamIn->streamFormat);
-#endif /* DEBUG */
 
     /* If the frequency of the device is different from the requested one we
      * need a converter. The same count if the number of channels is different. */
     if (   pStreamIn->deviceFormat.mSampleRate       != pStreamIn->streamFormat.mSampleRate
         || pStreamIn->deviceFormat.mChannelsPerFrame != pStreamIn->streamFormat.mChannelsPerFrame)
     {
-        err = AudioConverterNew(&pStreamIn->deviceFormat, &pStreamIn->streamFormat, &pStreamIn->converter);
+        LogRel(("CoreAudio: Input converter is active\n"));
+
+        err = AudioConverterNew(&pStreamIn->deviceFormat, &pStreamIn->streamFormat, &pStreamIn->pConverter);
         if (RT_UNLIKELY(err != noErr))
         {
-            LogRel(("CoreAudio: Failed to create the audio converte(%RI32). Input Format=%d, Output Foramt=%d\n",
-                     err, pStreamIn->deviceFormat, pStreamIn->streamFormat));
+            LogRel(("CoreAudio: Failed to create the audio converter (%RI32)\n", err));
             return VERR_AUDIO_BACKEND_INIT_FAILED;
         }
 
@@ -885,22 +897,13 @@ static int drvHostCoreAudioInitInput(PPDMAUDIOHSTSTRMIN pHstStrmIn, uint32_t *pc
              */
             const SInt32 channelMap[2] = {0, 0}; /* Channel map for mono -> stereo, */
 
-            err = AudioConverterSetProperty(pStreamIn->converter, kAudioConverterChannelMap, sizeof(channelMap), channelMap);
+            err = AudioConverterSetProperty(pStreamIn->pConverter, kAudioConverterChannelMap, sizeof(channelMap), channelMap);
             if (err != noErr)
             {
-                LogRel(("CoreAudio: Failed to set channel mapping for the audio input converter (%RI32)\n", err));
+                LogRel(("CoreAudio: Failed to set channel mapping (mono -> stereo) for the audio input converter (%RI32)\n", err));
                 return VERR_AUDIO_BACKEND_INIT_FAILED;
             }
         }
-
-        /* Set the new input format description for the stream. */
-        err = AudioUnitSetProperty(pStreamIn->audioUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input,
-                                   1, &pStreamIn->streamFormat, sizeof(pStreamIn->streamFormat));
-        if (err != noErr)
-        {
-            LogRel(("CoreAudio: Failed to set input format for input stream (%RI32)\n", err));
-            return VERR_AUDIO_BACKEND_INIT_FAILED;
-        }
 #if 0
         /* Set sample rate converter quality to maximum */
         uFlag = kAudioConverterQuality_Max;
@@ -909,17 +912,42 @@ static int drvHostCoreAudioInitInput(PPDMAUDIOHSTSTRMIN pHstStrmIn, uint32_t *pc
         if (err != noErr)
             LogRel(("CoreAudio: Failed to set input audio converter quality to the maximum (%RI32)\n", err));
 #endif
-        LogRel(("CoreAudio: Input converter is active\n"));
+
+        /* Set the new format description for the stream. */
+        err = AudioUnitSetProperty(pStreamIn->audioUnit,
+                                   kAudioUnitProperty_StreamFormat,
+                                   kAudioUnitScope_Output,
+                                   1,
+                                   &pStreamIn->deviceFormat,
+                                   sizeof(pStreamIn->deviceFormat));
+        if (RT_UNLIKELY(err != noErr))
+        {
+            LogRel(("CoreAudio: Failed to set input stream output format (%RI32)\n", err));
+            return VERR_AUDIO_BACKEND_INIT_FAILED;
+        }
+
+        err = AudioUnitSetProperty(pStreamIn->audioUnit,
+                                   kAudioUnitProperty_StreamFormat,
+                                   kAudioUnitScope_Input,
+                                   1,
+                                   &pStreamIn->deviceFormat,
+                                   sizeof(pStreamIn->deviceFormat));
+        if (RT_UNLIKELY(err != noErr))
+        {
+            LogRel(("CoreAudio: Failed to set stream input format (%RI32)\n", err));
+            return VERR_AUDIO_BACKEND_INIT_FAILED;
+        }
     }
     else
     {
-        /* Set the new output format description for the stream. */
+
+        /* Set the new output format description for the input stream. */
         err = AudioUnitSetProperty(pStreamIn->audioUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Output,
                                    1, &pStreamIn->streamFormat, sizeof(pStreamIn->streamFormat));
         if (err != noErr)
         {
             LogRel(("CoreAudio: Failed to set output format for input stream (%RI32)\n", err));
-            return VERR_AUDIO_BACKEND_INIT_FAILED; /** @todo Fudge! */
+            return VERR_AUDIO_BACKEND_INIT_FAILED;
         }
     }
 
@@ -966,11 +994,16 @@ static int drvHostCoreAudioInitInput(PPDMAUDIOHSTSTRMIN pHstStrmIn, uint32_t *pc
         return VERR_AUDIO_BACKEND_INIT_FAILED;
     }
 
+    /* Destroy any former internal ring buffer. */
+    if (pStreamIn->pBuf)
+    {
+        RTCircBufDestroy(pStreamIn->pBuf);
+        pStreamIn->pBuf = NULL;
+    }
+
     /* Calculate the ratio between the device and the stream sample rate. */
     pStreamIn->sampleRatio = pStreamIn->streamFormat.mSampleRate / pStreamIn->deviceFormat.mSampleRate;
 
-    /* Set to zero first */
-    pStreamIn->pBuf = NULL;
     /* Create the AudioBufferList structure with one buffer. */
     pStreamIn->bufferList.mNumberBuffers = 1;
     /* Initialize the buffer to nothing. */
@@ -1195,10 +1228,8 @@ static int drvHostCoreAudioInitOutput(PPDMAUDIOHSTSTRMOUT pHstStrmOut, uint32_t
     /* Create an AudioStreamBasicDescription based on our required audio settings. */
     drvHostCoreAudioPCMInfoToASBDesc(&pStreamOut->streamOut.Props, &pStreamOut->streamFormat);
 
-#ifdef DEBUG
     drvHostCoreAudioPrintASBDesc("CoreAudio: Output device", &pStreamOut->deviceFormat);
     drvHostCoreAudioPrintASBDesc("CoreAudio: Output format", &pStreamOut->streamFormat);
-#endif /* DEBUG */
 
     /* Set the new output format description for the stream. */
     err = AudioUnitSetProperty(pStreamOut->audioUnit, kAudioUnitProperty_StreamFormat, kAudioUnitScope_Input,
@@ -1271,6 +1302,13 @@ static int drvHostCoreAudioInitOutput(PPDMAUDIOHSTSTRMOUT pHstStrmOut, uint32_t
         rc = VERR_INVALID_PARAMETER;
     }
 
+    /* Destroy any former internal ring buffer. */
+    if (pStreamOut->pBuf)
+    {
+        RTCircBufDestroy(pStreamOut->pBuf);
+        pStreamOut->pBuf = NULL;
+    }
+
     /* Create the internal ring buffer. */
     rc = RTCircBufCreate(&pStreamOut->pBuf, cSamples << pHstStrmOut->Props.cShift);
     if (RT_SUCCESS(rc))
@@ -1375,8 +1413,12 @@ static DECLCALLBACK(int) drvHostCoreAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
             }
 
             rc = AudioMixBufWriteCirc(&pHstStrmIn->MixBuf, puBuf, cbToRead, &cWritten);
-            if (RT_FAILURE(rc))
+            if (   RT_FAILURE(rc)
+                || !cWritten)
+            {
+                RTCircBufReleaseReadBlock(pStreamIn->pBuf, cbToRead);
                 break;
+            }
 
             cbWritten = AUDIOMIXBUF_S2B(&pHstStrmIn->MixBuf, cWritten);
 
@@ -1388,7 +1430,7 @@ static DECLCALLBACK(int) drvHostCoreAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
             cbWrittenTotal += cbWritten;
         }
 
-        LogFlowFunc(("cbToWrite=%zu, cbWrittenTotal=%RU32\n", cbToWrite, cbWrittenTotal));
+        LogFlowFunc(("cbToWrite=%zu, cbToRead=%zu, cbWrittenTotal=%RU32, rc=%Rrc\n", cbToWrite, cbToRead, cbWrittenTotal, rc));
     }
     while (0);
 
@@ -1399,7 +1441,7 @@ static DECLCALLBACK(int) drvHostCoreAudioCaptureIn(PPDMIHOSTAUDIO pInterface, PP
         if (cWrittenTotal)
             rc = AudioMixBufMixToParent(&pHstStrmIn->MixBuf, cWrittenTotal, &cCaptured);
 
-        LogFlowFunc(("cWrittenTotal=%RU32 (%RU32 bytes), cCaptured, rc=%Rrc\n", cWrittenTotal, cbWrittenTotal, cCaptured, rc));
+        LogFlowFunc(("cWrittenTotal=%RU32 (%RU32 bytes), cCaptured=%RU32, rc=%Rrc\n", cWrittenTotal, cbWrittenTotal, cCaptured, rc));
 
         if (pcSamplesCaptured)
             *pcSamplesCaptured = cCaptured;
@@ -1498,9 +1540,15 @@ static DECLCALLBACK(int) drvHostCoreAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
 
     PCOREAUDIOSTREAMOUT pStreamOut = (PCOREAUDIOSTREAMOUT)pHstStrmOut;
 
+    int rc = VINF_SUCCESS;
+
     /* Check if the audio device should be reinitialized. If so do it. */
     if (ASMAtomicReadU32(&pStreamOut->status) == CA_STATUS_REINIT)
-        drvHostCoreAudioReinitOutput(pInterface, &pStreamOut->streamOut);
+    {
+        rc = drvHostCoreAudioReinitOutput(pInterface, &pStreamOut->streamOut);
+        if (RT_FAILURE(rc))
+            return rc;
+    }
 
     /* Not much else to do here. */
 
@@ -1512,7 +1560,6 @@ static DECLCALLBACK(int) drvHostCoreAudioPlayOut(PPDMIHOSTAUDIO pInterface, PPDM
         return VINF_SUCCESS;
     }
 
-    int rc = VINF_SUCCESS;
     uint32_t cbReadTotal = 0;
     uint32_t cAvail = AudioMixBufAvail(&pHstStrmOut->MixBuf);
     size_t cbAvail  = AUDIOMIXBUF_S2B(&pHstStrmOut->MixBuf, cAvail);
@@ -1779,10 +1826,10 @@ static DECLCALLBACK(int) drvHostCoreAudioFiniIn(PPDMIHOSTAUDIO pInterface, PPDMA
                 LogRel(("CoreAudio: [Output] Failed to remove the default input device changed listener (%RI32)\n", err));
         }
 
-        if (pStreamIn->converter)
+        if (pStreamIn->pConverter)
         {
-            AudioConverterDispose(pStreamIn->converter);
-            pStreamIn->converter = NULL;
+            AudioConverterDispose(pStreamIn->pConverter);
+            pStreamIn->pConverter = NULL;
         }
 
         err = AudioUnitUninitialize(pStreamIn->audioUnit);
@@ -1791,13 +1838,15 @@ static DECLCALLBACK(int) drvHostCoreAudioFiniIn(PPDMIHOSTAUDIO pInterface, PPDMA
             err = CloseComponent(pStreamIn->audioUnit);
             if (RT_LIKELY(err == noErr))
             {
-                RTCircBufDestroy(pStreamIn->pBuf);
-
-                pStreamIn->audioUnit   = NULL;
-                pStreamIn->deviceID    = kAudioDeviceUnknown;
-                pStreamIn->pBuf        = NULL;
-                pStreamIn->sampleRatio = 1;
-                pStreamIn->rpos        = 0;
+                pStreamIn->deviceID      = kAudioDeviceUnknown;
+                pStreamIn->audioUnit     = NULL;
+                pStreamIn->offBufferRead = 0;
+                pStreamIn->sampleRatio   = 1;
+                if (pStreamIn->pBuf)
+                {
+                    RTCircBufDestroy(pStreamIn->pBuf);
+                    pStreamIn->pBuf = NULL;
+                }
 
                 ASMAtomicXchgU32(&pStreamIn->status, CA_STATUS_UNINIT);
             }
@@ -1887,11 +1936,13 @@ static DECLCALLBACK(int) drvHostCoreAudioFiniOut(PPDMIHOSTAUDIO pInterface, PPDM
             err = CloseComponent(pStreamOut->audioUnit);
             if (err == noErr)
             {
-                RTCircBufDestroy(pStreamOut->pBuf);
-                pStreamOut->pBuf      = NULL;
-
-                pStreamOut->audioUnit = NULL;
                 pStreamOut->deviceID  = kAudioDeviceUnknown;
+                pStreamOut->audioUnit = NULL;
+                if (pStreamOut->pBuf)
+                {
+                    RTCircBufDestroy(pStreamOut->pBuf);
+                    pStreamOut->pBuf = NULL;
+                }
 
                 ASMAtomicXchgU32(&pStreamOut->status, CA_STATUS_UNINIT);
             }
@@ -1915,17 +1966,19 @@ static DECLCALLBACK(int) drvHostCoreAudioInitIn(PPDMIHOSTAUDIO pInterface,
 {
     PCOREAUDIOSTREAMIN pStreamIn = (PCOREAUDIOSTREAMIN)pHstStrmIn;
 
-    LogFlowFunc(("enmRecSource=%ld\n"));
+    LogFlowFunc(("enmRecSource=%ld\n", enmRecSource));
 
-    ASMAtomicXchgU32(&pStreamIn->status, CA_STATUS_UNINIT);
+    pStreamIn->deviceID                  = kAudioDeviceUnknown;
+    pStreamIn->audioUnit                 = NULL;
+    pStreamIn->pConverter                = NULL;
+    pStreamIn->bufferList.mNumberBuffers = 0;
+    pStreamIn->offBufferRead             = 0;
+    pStreamIn->sampleRatio               = 1;
+    pStreamIn->pBuf                      = NULL;
+    pStreamIn->status                    = CA_STATUS_UNINIT;
+    pStreamIn->fDefDevChgListReg         = false;
 
-    pStreamIn->audioUnit   = NULL;
-    pStreamIn->deviceID    = kAudioDeviceUnknown;
-    pStreamIn->converter   = NULL;
-    pStreamIn->sampleRatio = 1;
-    pStreamIn->rpos        = 0;
-
-    bool fDeviceByUser = false;
+    bool fDeviceByUser = false; /* Do we use a device which was set by the user? */
 
     /* Initialize the hardware info section with the audio settings */
     int rc = DrvAudioStreamCfgToProps(pCfg, &pStreamIn->streamIn.Props);
@@ -1975,56 +2028,58 @@ static DECLCALLBACK(int) drvHostCoreAudioInitOut(PPDMIHOSTAUDIO pInterface,
 {
     PCOREAUDIOSTREAMOUT pStreamOut = (PCOREAUDIOSTREAMOUT)pHstStrmOut;
 
-    OSStatus err = noErr;
-    int rc = 0;
-    bool fDeviceByUser = false; /* use we a device which was set by the user? */
-
     LogFlowFuncEnter();
 
-    ASMAtomicXchgU32(&pStreamOut->status, CA_STATUS_UNINIT);
+    pStreamOut->deviceID                  = kAudioDeviceUnknown;
+    pStreamOut->audioUnit                 = NULL;
+    pStreamOut->pBuf                      = NULL;
+    pStreamOut->status                    = CA_STATUS_UNINIT;
+    pStreamOut->fDefDevChgListReg         = false;
 
-    pStreamOut->audioUnit = NULL;
-    pStreamOut->deviceID  = kAudioDeviceUnknown;
+    bool fDeviceByUser = false; /* Do we use a device which was set by the user? */
 
     /* Initialize the hardware info section with the audio settings */
-    DrvAudioStreamCfgToProps(pCfg, &pStreamOut->streamOut.Props);
-
+    int rc = DrvAudioStreamCfgToProps(pCfg, &pStreamOut->streamOut.Props);
+    if (RT_SUCCESS(rc))
+    {
 #if 0
-    /* Try to find the audio device set by the user. Use
-     * export VBOX_COREAUDIO_OUTPUT_DEVICE_UID=AppleHDAEngineOutput:0
-     * to set it. */
-    if (DeviceUID.pszOutputDeviceUID)
-    {
-        pStreamOut->audioDeviceId = drvHostCoreAudioDeviceUIDtoID(DeviceUID.pszOutputDeviceUID);
-        /* Not fatal */
-        if (pStreamOut->audioDeviceId == kAudioDeviceUnknown)
-            LogRel(("CoreAudio: Unable to find output device %s. Falling back to the default audio device. \n", DeviceUID.pszOutputDeviceUID));
-        else
-            fDeviceByUser = true;
-    }
-#endif
-
-    rc = drvHostCoreAudioInitOutput(pHstStrmOut, pcSamples);
-    if (RT_FAILURE(rc))
-        return rc;
-
-    /* When the devices isn't forced by the user, we want default device change notifications. */
-    if (!fDeviceByUser)
-    {
-        AudioObjectPropertyAddress propAdr = { kAudioHardwarePropertyDefaultOutputDevice, kAudioObjectPropertyScopeGlobal,
-                                               kAudioObjectPropertyElementMaster };
-        err = AudioObjectAddPropertyListener(kAudioObjectSystemObject, &propAdr,
-                                             drvHostCoreAudioDefaultDeviceChanged, (void *)pStreamOut);
-        /* Not fatal. */
-        if (RT_LIKELY(err == noErr))
+        /* Try to find the audio device set by the user. Use
+         * export VBOX_COREAUDIO_OUTPUT_DEVICE_UID=AppleHDAEngineOutput:0
+         * to set it. */
+        if (DeviceUID.pszOutputDeviceUID)
         {
-            pStreamOut->fDefDevChgListReg = true;
+            pStreamOut->audioDeviceId = drvHostCoreAudioDeviceUIDtoID(DeviceUID.pszOutputDeviceUID);
+            /* Not fatal */
+            if (pStreamOut->audioDeviceId == kAudioDeviceUnknown)
+                LogRel(("CoreAudio: Unable to find output device %s. Falling back to the default audio device. \n", DeviceUID.pszOutputDeviceUID));
+            else
+                fDeviceByUser = true;
         }
-        else
-            LogRel(("CoreAudio: Failed to add the default output device changed listener (%RI32)\n", err));
+#endif
+        rc = drvHostCoreAudioInitOutput(pHstStrmOut, pcSamples);
     }
 
-    return VINF_SUCCESS;
+    if (RT_SUCCESS(rc))
+    {
+        /* When the devices isn't forced by the user, we want default device change notifications. */
+        if (!fDeviceByUser)
+        {
+            AudioObjectPropertyAddress propAdr = { kAudioHardwarePropertyDefaultOutputDevice, kAudioObjectPropertyScopeGlobal,
+                                                   kAudioObjectPropertyElementMaster };
+            OSStatus err = AudioObjectAddPropertyListener(kAudioObjectSystemObject, &propAdr,
+                                                          drvHostCoreAudioDefaultDeviceChanged, (void *)pStreamOut);
+            /* Not fatal. */
+            if (RT_LIKELY(err == noErr))
+            {
+                pStreamOut->fDefDevChgListReg = true;
+            }
+            else
+                LogRel(("CoreAudio: Failed to add the default output device changed listener (%RI32)\n", err));
+        }
+    }
+
+    LogFlowFuncLeaveRC(rc);
+    return rc;
 }
 
 static DECLCALLBACK(bool) drvHostCoreAudioIsEnabled(PPDMIHOSTAUDIO pInterface, PDMAUDIODIR enmDir)
