@@ -1,7 +1,6 @@
+/* $Id: UIMachineWindow.h $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIMachineWindow class declaration
+ * VBox Qt GUI - UIMachineWindow class declaration.
  */
 
 /*
@@ -24,7 +23,10 @@
 
 /* GUI includes: */
 #include "QIWithRetranslateUI.h"
-#include "UIDefs.h"
+#include "UIExtraDataDefs.h"
+#ifdef Q_WS_MAC
+# include "VBoxUtils-darwin.h"
+#endif /* Q_WS_MAC */
 
 /* COM includes: */
 #include "COMEnums.h"
@@ -38,12 +40,18 @@ class CSession;
 class UISession;
 class UIMachineLogic;
 class UIMachineView;
+class UIActionPool;
 
 /* Machine-window interface: */
 class UIMachineWindow : public QIWithRetranslateUI2<QMainWindow>
 {
     Q_OBJECT;
 
+signals:
+
+    /** Notifies about frame-buffer resize. */
+    void sigFrameBufferResize();
+
 public:
 
     /* Factory functions to create/destroy machine-window: */
@@ -55,22 +63,43 @@ public:
     void cleanup();
 
     /* Public getters: */
+    ulong screenId() const { return m_uScreenId; }
     UIMachineView* machineView() const { return m_pMachineView; }
     UIMachineLogic* machineLogic() const { return m_pMachineLogic; }
+    UIActionPool* actionPool() const;
     UISession* uisession() const;
-    CSession& session() const;
-    CMachine machine() const;
 
-    /** Adjusts machine-window size to correspond current guest screen size.
+    /** Returns the session reference. */
+    CSession& session() const;
+    /** Returns the session's machine reference. */
+    CMachine& machine() const;
+    /** Returns the session's console reference. */
+    CConsole& console() const;
+
+    /** Returns the machine name. */
+    const QString& machineName() const;
+
+    /** Adjusts machine-window size to correspond current machine-view size.
       * @param fAdjustPosition determines whether is it necessary to adjust position too.
       * @note  Reimplemented in sub-classes. Base implementation does nothing. */
     virtual void normalizeGeometry(bool fAdjustPosition) { Q_UNUSED(fAdjustPosition); }
 
+    /** Adjusts machine-view size to correspond current machine-window size. */
+    virtual void adjustMachineViewSize();
+
+    /** Sends machine-view size-hint to the guest. */
+    virtual void sendMachineViewSizeHint();
+
+#ifdef VBOX_WITH_MASKED_SEAMLESS
     /* Virtual caller for base class setMask: */
     virtual void setMask(const QRegion &region);
+#endif /* VBOX_WITH_MASKED_SEAMLESS */
 
 protected slots:
 
+    /** Performs machine-window activation. */
+    void sltActivateWindow() { activateWindow(); }
+
     /* Session event-handlers: */
     virtual void sltMachineStateChanged();
 
@@ -87,9 +116,26 @@ protected:
 
     /* Event handlers: */
 #ifdef Q_WS_X11
+    /** X11: Native event handler. */
     bool x11Event(XEvent *pEvent);
 #endif /* Q_WS_X11 */
-    void closeEvent(QCloseEvent *pEvent);
+
+    /** Show event handler. */
+    void showEvent(QShowEvent *pShowEvent);
+
+    /** Close event handler. */
+    void closeEvent(QCloseEvent *pCloseEvent);
+
+#ifdef Q_WS_MAC
+    /** Mac OS X: Handles native notifications.
+      * @param  strNativeNotificationName  Native notification name. */
+    virtual void handleNativeNotification(const QString & /* strNativeNotificationName */) {}
+
+    /** Mac OS X: Handles standard window button callbacks.
+      * @param  enmButtonType   Brings standard window button type.
+      * @param  fWithOptionKey  Brings whether the Option key was held. */
+    virtual void handleStandardWindowButtonCallback(StandardWindowButtonType enmButtonType, bool fWithOptionKey);
+#endif /* Q_WS_MAC */
 
     /* Prepare helpers: */
     virtual void prepareSessionConnections();
@@ -109,11 +155,7 @@ protected:
     virtual void cleanupStatusBar() {}
     virtual void cleanupMenu() {}
     virtual void cleanupMainLayout() {}
-    virtual void cleanupSessionConnections() {}
-
-    /* Visibility stuff: */
-    void handleScreenCountChange();
-    void handleScreenGeometryChange();
+    virtual void cleanupSessionConnections();
 
     /* Update stuff: */
     virtual void updateAppearanceOf(int iElement);
@@ -123,9 +165,21 @@ protected:
 
     /* Helpers: */
     const QString& defaultWindowTitle() const { return m_strWindowTitlePrefix; }
-    static Qt::WindowFlags windowFlags(UIVisualStateType visualStateType);
     static Qt::Alignment viewAlignment(UIVisualStateType visualStateType);
 
+#ifdef Q_WS_MAC
+    /** Mac OS X: Handles native notifications.
+      * @param  strNativeNotificationName  Native notification name.
+      * @param  pWidget                    Widget, notification related to. */
+    static void handleNativeNotification(const QString &strNativeNotificationName, QWidget *pWidget);
+
+    /** Mac OS X: Handles standard window button callbacks.
+      * @param  enmButtonType   Brings standard window button type.
+      * @param  fWithOptionKey  Brings whether the Option key was held.
+      * @param  pWidget         Brings widget, callback related to. */
+    static void handleStandardWindowButtonCallback(StandardWindowButtonType enmButtonType, bool fWithOptionKey, QWidget *pWidget);
+#endif /* Q_WS_MAC */
+
     /* Variables: */
     UIMachineLogic *m_pMachineLogic;
     UIMachineView *m_pMachineView;
