@@ -1,8 +1,6 @@
 /* $Id: UIGDetailsSet.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIGDetailsSet class implementation
+ * VBox Qt GUI - UIGDetailsSet class implementation.
  */
 
 /*
@@ -17,20 +15,30 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include <precomp.h>
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 /* GUI includes: */
-#include "UIGDetailsSet.h"
-#include "UIGDetailsModel.h"
-#include "UIGDetailsElements.h"
-#include "UIVMItem.h"
-#include "UIConverter.h"
-#include "UIVirtualBoxEventHandler.h"
-#include "VBoxGlobal.h"
+# include "UIGDetailsSet.h"
+# include "UIGDetailsModel.h"
+# include "UIGDetailsElements.h"
+# include "UIVMItem.h"
+# include "UIVirtualBoxEventHandler.h"
+# include "VBoxGlobal.h"
 
 /* COM includes: */
-#include "CUSBController.h"
+# include "CUSBController.h"
+# include "CUSBDeviceFilters.h"
+
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 
 UIGDetailsSet::UIGDetailsSet(UIGDetailsItem *pParent)
     : UIGDetailsItem(pParent)
+    , m_pMachineItem(0)
+    , m_fHasDetails(false)
+    , m_configurationAccessLevel(ConfigurationAccessLevel_Null)
     , m_fFullSet(true)
     , m_pBuildStep(0)
     , m_iLastStepNumber(-1)
@@ -54,18 +62,22 @@ UIGDetailsSet::~UIGDetailsSet()
     parentItem()->removeItem(this);
 }
 
-void UIGDetailsSet::buildSet(const CMachine &machine, bool fFullSet, const QStringList &settings)
+void UIGDetailsSet::buildSet(UIVMItem *pMachineItem, bool fFullSet, const QMap<DetailsElementType, bool> &settings)
 {
     /* Remember passed arguments: */
-    m_machine = machine;
+    m_pMachineItem = pMachineItem;
+    m_machine = m_pMachineItem->machine();
+    m_fHasDetails = m_pMachineItem->hasDetails();
     m_fFullSet = fFullSet;
     m_settings = settings;
 
     /* Cleanup superfluous items: */
-    if (!m_fFullSet)
+    if (!m_fFullSet || !m_fHasDetails)
     {
+        int iFirstItem = m_fHasDetails ? DetailsElementType_Display : DetailsElementType_General;
+        int iLastItem = DetailsElementType_Description;
         bool fCleanupPerformed = false;
-        for (int i = DetailsElementType_Display; i <= DetailsElementType_Description; ++i)
+        for (int i = iFirstItem; i <= iLastItem; ++i)
             if (m_elements.contains(i))
             {
                 delete m_elements[i];
@@ -75,18 +87,23 @@ void UIGDetailsSet::buildSet(const CMachine &machine, bool fFullSet, const QStri
             updateGeometry();
     }
 
+    /* Make sure we have details: */
+    if (!m_fHasDetails)
+    {
+        /* Reset last-step number: */
+        m_iLastStepNumber = -1;
+        /* Notify parent group we are built: */
+        emit sigBuildDone();
+        return;
+    }
+
     /* Choose last-step number: */
     m_iLastStepNumber = m_fFullSet ? DetailsElementType_Description : DetailsElementType_Preview;
 
     /* Fetch USB controller restrictions: */
-    const CUSBController &ctl = m_machine.GetUSBController();
-    if (ctl.isNull() || !ctl.GetProxyAvailable())
-    {
-        QString strElementTypeOpened = gpConverter->toInternalString(DetailsElementType_USB);
-        QString strElementTypeClosed = strElementTypeOpened + "Closed";
-        m_settings.removeAll(strElementTypeOpened);
-        m_settings.removeAll(strElementTypeClosed);
-    }
+    const CUSBDeviceFilters &filters = m_machine.GetUSBDeviceFilters();
+    if (filters.isNull() || !m_machine.GetUSBProxyAvailable())
+        m_settings.remove(DetailsElementType_USB);
 
     /* Start building set: */
     rebuildSet();
@@ -107,12 +124,10 @@ void UIGDetailsSet::sltBuildStep(QString strStepId, int iStepNumber)
     {
         /* Load details settings: */
         DetailsElementType elementType = (DetailsElementType)iStepNumber;
-        QString strElementTypeOpened = gpConverter->toInternalString(elementType);
-        QString strElementTypeClosed = strElementTypeOpened + "Closed";
         /* Should the element be visible? */
-        bool fVisible = m_settings.contains(strElementTypeOpened) || m_settings.contains(strElementTypeClosed);
+        bool fVisible = m_settings.contains(elementType);
         /* Should the element be opened? */
-        bool fOpen = m_settings.contains(strElementTypeOpened);
+        bool fOpen = fVisible && m_settings[elementType];
 
         /* Check if element is present already: */
         UIGDetailsElement *pElement = element(elementType);
@@ -185,10 +200,6 @@ void UIGDetailsSet::sltMachineStateChange(QString strId)
     if (m_machine.GetId() != strId)
         return;
 
-    /* Update hover accessibility: */
-    foreach (UIGDetailsItem *pItem, items())
-        pItem->toElement()->updateHoverAccessibility();
-
     /* Update appearance: */
     rebuildSet();
 }
@@ -329,15 +340,22 @@ void UIGDetailsSet::prepareConnections()
     connect(gVBoxEvents, SIGNAL(sigMachineStateChange(QString, KMachineState)), this, SLOT(sltMachineStateChange(QString)));
     connect(gVBoxEvents, SIGNAL(sigMachineDataChange(QString)), this, SLOT(sltMachineAttributesChange(QString)));
     connect(gVBoxEvents, SIGNAL(sigSessionStateChange(QString, KSessionState)), this, SLOT(sltMachineAttributesChange(QString)));
+    connect(gVBoxEvents, SIGNAL(sigSnapshotTake(QString, QString)), this, SLOT(sltMachineAttributesChange(QString)));
+    connect(gVBoxEvents, SIGNAL(sigSnapshotDelete(QString, QString)), this, SLOT(sltMachineAttributesChange(QString)));
     connect(gVBoxEvents, SIGNAL(sigSnapshotChange(QString, QString)), this, SLOT(sltMachineAttributesChange(QString)));
+    connect(gVBoxEvents, SIGNAL(sigSnapshotRestore(QString, QString)), this, SLOT(sltMachineAttributesChange(QString)));
 
     /* Meidum-enumeration connections: */
-    connect(&vboxGlobal(), SIGNAL(mediumEnumStarted()), this, SLOT(sltUpdateAppearance()));
-    connect(&vboxGlobal(), SIGNAL(mediumEnumFinished(const VBoxMediaList &)), this, SLOT(sltUpdateAppearance()));
+    connect(&vboxGlobal(), SIGNAL(sigMediumEnumerationStarted()), this, SLOT(sltUpdateAppearance()));
+    connect(&vboxGlobal(), SIGNAL(sigMediumEnumerationFinished()), this, SLOT(sltUpdateAppearance()));
 }
 
 int UIGDetailsSet::minimumWidthHint() const
 {
+    /* Zero if has no details: */
+    if (!hasDetails())
+        return 0;
+
     /* Prepare variables: */
     int iMargin = data(SetData_Margin).toInt();
     int iSpacing = data(SetData_Spacing).toInt();
@@ -366,6 +384,7 @@ int UIGDetailsSet::minimumWidthHint() const
 #endif /* VBOX_WITH_PARALLEL_PORTS */
             case DetailsElementType_USB:
             case DetailsElementType_SF:
+            case DetailsElementType_UI:
             case DetailsElementType_Description:
             {
                 iMinimumWidthHint = qMax(iMinimumWidthHint, pItem->minimumWidthHint());
@@ -393,6 +412,10 @@ int UIGDetailsSet::minimumWidthHint() const
 
 int UIGDetailsSet::minimumHeightHint() const
 {
+    /* Zero if has no details: */
+    if (!hasDetails())
+        return 0;
+
     /* Prepare variables: */
     int iMargin = data(SetData_Margin).toInt();
     int iSpacing = data(SetData_Spacing).toInt();
@@ -421,6 +444,7 @@ int UIGDetailsSet::minimumHeightHint() const
 #endif /* VBOX_WITH_PARALLEL_PORTS */
             case DetailsElementType_USB:
             case DetailsElementType_SF:
+            case DetailsElementType_UI:
             case DetailsElementType_Description:
             {
                 iMinimumHeightHint += (pItem->minimumHeightHint() + iSpacing);
@@ -475,6 +499,7 @@ void UIGDetailsSet::updateLayout()
 #endif /* VBOX_WITH_PARALLEL_PORTS */
             case DetailsElementType_USB:
             case DetailsElementType_SF:
+            case DetailsElementType_UI:
             case DetailsElementType_Description:
             {
                 /* Move element: */
@@ -491,8 +516,6 @@ void UIGDetailsSet::updateLayout()
                 {
                     /* Resize element to required width: */
                     pElement->resize(iWidth, pElement->geometry().height());
-                    /* Update minimum-height-hint: */
-                    pElement->updateMinimumTextHeight();
                 }
                 /* Acquire required height: */
                 int iHeight = pElement->minimumHeightHint();
@@ -529,6 +552,13 @@ void UIGDetailsSet::updateLayout()
 
 void UIGDetailsSet::rebuildSet()
 {
+    /* Make sure we have details: */
+    if (!m_fHasDetails)
+        return;
+
+    /* Recache properties: */
+    m_configurationAccessLevel = m_pMachineItem->configurationAccessLevel();
+
     /* Cleanup build-step: */
     delete m_pBuildStep;
     m_pBuildStep = 0;
@@ -558,6 +588,7 @@ UIGDetailsElement* UIGDetailsSet::createElement(DetailsElementType elementType,
 #endif /* VBOX_WITH_PARALLEL_PORTS */
         case DetailsElementType_USB:         return new UIGDetailsElementUSB(this, fOpen);
         case DetailsElementType_SF:          return new UIGDetailsElementSF(this, fOpen);
+        case DetailsElementType_UI:          return new UIGDetailsElementUI(this, fOpen);
         case DetailsElementType_Description: return new UIGDetailsElementDescription(this, fOpen);
     }
     return 0;
