@@ -40,7 +40,7 @@ WARNING: The vboxdrv kernel module is not loaded. Either there is no module
          available for the current kernel (`uname -r`) or it failed to
          load. Please recompile the kernel module and install it by
 
-           sudo /sbin/vboxconfig
+           sudo /sbin/rcvboxdrv setup
 
          You will not be able to start VMs until this problem is fixed.
 EOF
@@ -48,7 +48,7 @@ elif [ ! -c /dev/vboxdrv ]; then
     cat << EOF
 WARNING: The character device /dev/vboxdrv does not exist. Try
 
-           sudo /sbin/vboxconfig
+           sudo /sbin/rcvboxdrv setup
 
          and if that is not successful, try to re-install the package.
 
