@@ -1224,17 +1224,20 @@ USBProxyService::initFilterFromDevice(PUSBFILTER aFilter, HostUSBDevice *aDevice
     vrc = USBFilterSetNumExact(aFilter, USBFILTERIDX_BUS,               pDev->bBus,             false); AssertRC(vrc);
     if (pDev->pszSerialNumber)
     {
-        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_SERIAL_NUMBER_STR, pDev->pszSerialNumber, true);
+        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_SERIAL_NUMBER_STR, pDev->pszSerialNumber,
+                                      true /*fMustBePresent*/, false /*fPurge*/);
         AssertRC(vrc);
     }
     if (pDev->pszProduct)
     {
-        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_PRODUCT_STR, pDev->pszProduct, true);
+        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_PRODUCT_STR, pDev->pszProduct,
+                                      true /*fMustBePresent*/, false /*fPurge*/);
         AssertRC(vrc);
     }
     if (pDev->pszManufacturer)
     {
-        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_MANUFACTURER_STR, pDev->pszManufacturer, true);
+        vrc = USBFilterSetStringExact(aFilter, USBFILTERIDX_MANUFACTURER_STR, pDev->pszManufacturer,
+                                      true /*fMustBePresent*/, false /*fPurge*/);
         AssertRC(vrc);
     }
 }
