@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Oracle Corporation
+ * Copyright (C) 2006-2015 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,9 +24,10 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-/*******************************************************************************
-*   Defined Constants And Macros                                               *
-*******************************************************************************/
+
+/*********************************************************************************************************************************
+*   Defined Constants And Macros                                                                                                 *
+*********************************************************************************************************************************/
 /** Enables the use of POSIX RT timers. */
 #ifndef RT_OS_SOLARIS /* Solaris 10 doesn't have SIGEV_THREAD */
 # define IPRT_WITH_POSIX_TIMERS
@@ -43,9 +44,9 @@
 #endif
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #define LOG_GROUP   RTLOGGROUP_TIMER
 #include <iprt/timer.h>
 #include <iprt/alloc.h>
@@ -57,6 +58,7 @@
 #include <iprt/string.h>
 #include <iprt/once.h>
 #include <iprt/err.h>
+#include <iprt/initterm.h>
 #include <iprt/critsect.h>
 #include "internal/magics.h"
 
@@ -72,9 +74,9 @@
 #include <pthread.h>
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 #ifdef IPRT_WITH_POSIX_TIMERS
 /** Init the critsect on first call. */
 static RTONCE g_TimerOnce = RTONCE_INITIALIZER;
@@ -93,9 +95,9 @@ static RTTHREAD g_TimerThread;
 #endif /* IPRT_WITH_POSIX_TIMERS */
 
 
-/*******************************************************************************
-*   Structures and Typedefs                                                    *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Structures and Typedefs                                                                                                      *
+*********************************************************************************************************************************/
 /**
  * The internal representation of a timer handle.
  */
@@ -152,14 +154,12 @@ typedef struct RTTIMER
  * RTOnce callback that initializes the critical section.
  *
  * @returns RTCritSectInit return code.
- * @param   pvUser1     NULL, ignored.
- * @param   pvUser2     NULL, ignored.
+ * @param   pvUser      NULL, ignored.
  *
  */
-static DECLCALLBACK(int) rtTimerOnce(void *pvUser1, void *pvUser2)
+static DECLCALLBACK(int) rtTimerOnce(void *pvUser)
 {
-    NOREF(pvUser1);
-    NOREF(pvUser2);
+    NOREF(pvUser);
     return RTCritSectInit(&g_TimerCritSect);
 }
 #endif
@@ -406,6 +406,13 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
     if (fFlags & RTTIMER_FLAGS_CPU_SPECIFIC)
         return VERR_NOT_SUPPORTED;
 
+    /*
+     * We need the signal masks to be set correctly, which they won't be in
+     * unobtrusive mode.
+     */
+    if (RTR3InitIsUnobtrusive())
+        return VERR_NOT_SUPPORTED;
+
 #ifndef IPRT_WITH_POSIX_TIMERS
     /*
      * Check if timer is busy.
@@ -539,7 +546,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, uint32_
     /*
      * Do the global init first.
      */
-    int rc = RTOnce(&g_TimerOnce, rtTimerOnce, NULL, NULL);
+    int rc = RTOnce(&g_TimerOnce, rtTimerOnce, NULL);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -662,7 +669,7 @@ RTR3DECL(int) RTTimerDestroy(PRTTIMER pTimer)
         TimerSpec.it_value.tv_sec     = 0;
         TimerSpec.it_value.tv_nsec    = 0;
         int err = timer_settime(pTimer->NativeTimer, 0, &TimerSpec, NULL); NOREF(err);
-        AssertMsg(!err, ("%d\n", err));
+        AssertMsg(!err, ("%d / %d\n", err, errno));
     }
 #endif
 
@@ -759,7 +766,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
     TimerSpec.it_interval.tv_sec  = pTimer->u64NanoInterval / 1000000000;
     TimerSpec.it_interval.tv_nsec = pTimer->u64NanoInterval % 1000000000;
     int err = timer_settime(pTimer->NativeTimer, 0, &TimerSpec, NULL);
-    int rc = RTErrConvertFromErrno(err);
+    int rc = err == 0 ? VINF_SUCCESS : RTErrConvertFromErrno(errno);
 #endif /* IPRT_WITH_POSIX_TIMERS */
 
     if (RT_FAILURE(rc))
@@ -806,7 +813,7 @@ RTDECL(int) RTTimerStop(PRTTIMER pTimer)
     TimerSpec.it_value.tv_sec     = 0;
     TimerSpec.it_value.tv_nsec    = 0;
     int err = timer_settime(pTimer->NativeTimer, 0, &TimerSpec, NULL);
-    int rc = RTErrConvertFromErrno(err);
+    int rc = err == 0 ? VINF_SUCCESS : RTErrConvertFromErrno(errno);
 #endif /* IPRT_WITH_POSIX_TIMERS */
 
     return rc;
